"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonWebKeyFormatter = void 0;
const dev_tunnels_ssh_1 = require("@microsoft/dev-tunnels-ssh");
const keyData_1 = require("./keyData");
/** Provides import/export of the JSON Web Key format. */
class JsonWebKeyFormatter {
    async import(keyData) {
        var _a, _b;
        if (!keyData)
            throw new TypeError('KeyData object expected.');
        let keyJson;
        try {
            keyJson = JSON.parse(keyData.data.toString('utf8'));
        }
        catch (e) {
            return null;
        }
        const jwk = keyJson;
        if (!jwk) {
            return null;
        }
        if (jwk.kty === 'RSA') {
            const parameters = JsonWebKeyFormatter.parseRsa(jwk);
            const keyPair = dev_tunnels_ssh_1.SshAlgorithms.publicKey.rsaWithSha512.createKeyPair();
            await keyPair.importParameters(parameters);
            keyPair.comment = (_a = jwk.comment) !== null && _a !== void 0 ? _a : null;
            return keyPair;
        }
        else if (jwk.kty === 'EC') {
            const parameters = JsonWebKeyFormatter.parseEC(jwk);
            const keyPair = new dev_tunnels_ssh_1.ECDsa.KeyPair();
            await keyPair.importParameters(parameters);
            keyPair.comment = (_b = jwk.comment) !== null && _b !== void 0 ? _b : null;
            return keyPair;
        }
        else {
            throw new Error(`Key type not supported: ${jwk.kty}`);
        }
    }
    async export(keyPair, includePrivate) {
        var _a;
        if (!keyPair)
            throw new TypeError('KeyPair object expected.');
        let jwk;
        if (keyPair instanceof dev_tunnels_ssh_1.Rsa.KeyPair) {
            const parameters = await keyPair.exportParameters();
            jwk = JsonWebKeyFormatter.formatRsa(parameters, includePrivate);
        }
        else if (keyPair instanceof dev_tunnels_ssh_1.ECDsa.KeyPair) {
            const parameters = await keyPair.exportParameters();
            jwk = JsonWebKeyFormatter.formatEC(parameters, includePrivate);
        }
        else {
            throw new Error('KeyPair class not supported: ' + ((_a = keyPair.constructor) === null || _a === void 0 ? void 0 : _a.name));
        }
        if (keyPair.comment) {
            jwk.comment = keyPair.comment;
        }
        const keyData = new keyData_1.KeyData();
        keyData.data = Buffer.from(JSON.stringify(jwk, null, '\t') + '\n', 'utf8');
        return keyData;
    }
    async decrypt(keyData, passphrase) {
        if (!keyData)
            throw new TypeError('KeyData object expected.');
        // Just check if it's valid JSON.
        try {
            JSON.parse(keyData.data.toString('utf8'));
            return keyData;
        }
        catch (e) {
            return null;
        }
    }
    encrypt(keyData, passphrase) {
        throw new Error('JWK does not support encryption.');
    }
    static formatRsa(rsa, includePrivate) {
        const formatBigInt = JsonWebKeyFormatter.formatBigInt;
        let jwk;
        if (includePrivate !== false && rsa.d && rsa.p && rsa.q && rsa.dp && rsa.dq && rsa.qi) {
            jwk = {
                kty: 'RSA',
                n: formatBigInt(rsa.modulus),
                e: formatBigInt(rsa.exponent),
                d: formatBigInt(rsa.d),
                p: formatBigInt(rsa.p),
                q: formatBigInt(rsa.q),
                dp: formatBigInt(rsa.dp),
                dq: formatBigInt(rsa.dq),
                qi: formatBigInt(rsa.qi),
            };
        }
        else if (!includePrivate) {
            jwk = {
                kty: 'RSA',
                n: formatBigInt(rsa.modulus),
                e: formatBigInt(rsa.exponent),
            };
        }
        else {
            throw new Error('Missing private key parameters.');
        }
        return jwk;
    }
    static parseRsa(jwk) {
        if ((jwk === null || jwk === void 0 ? void 0 : jwk.kty) !== 'RSA' || !(jwk.n && jwk.e))
            throw new Error('Invalid RSA JWK.');
        const parseBigInt = JsonWebKeyFormatter.parseBigInt;
        let rsa;
        if (jwk.d && jwk.p && jwk.q && jwk.dp && jwk.dq && jwk.qi) {
            rsa = {
                modulus: parseBigInt(jwk.n),
                exponent: parseBigInt(jwk.e),
                d: parseBigInt(jwk.d),
                p: parseBigInt(jwk.p),
                q: parseBigInt(jwk.q),
                dp: parseBigInt(jwk.dp),
                dq: parseBigInt(jwk.dq),
                qi: parseBigInt(jwk.qi),
            };
        }
        else {
            rsa = {
                modulus: parseBigInt(jwk.n),
                exponent: parseBigInt(jwk.e),
            };
        }
        return rsa;
    }
    static formatEC(ec, includePrivate) {
        const curve = dev_tunnels_ssh_1.ECDsa.curves.find((c) => c.oid === ec.curve.oid);
        const keySizeInBytes = Math.ceil(curve.keySize / 8);
        const formatBigInt = JsonWebKeyFormatter.formatBigInt;
        const jwk = {
            kty: 'EC',
            crv: ec.curve.name,
            x: formatBigInt(ec.x, keySizeInBytes),
            y: formatBigInt(ec.y, keySizeInBytes),
        };
        if (includePrivate !== false && ec.d) {
            jwk.d = formatBigInt(ec.d, keySizeInBytes);
        }
        else if (includePrivate) {
            throw new Error('Missing private key parameters.');
        }
        return jwk;
    }
    static parseEC(jwk) {
        var _a;
        if ((jwk === null || jwk === void 0 ? void 0 : jwk.kty) !== 'EC' || !(jwk.crv && jwk.x && jwk.y))
            throw new Error('Invalid EC JWK.');
        const curveOid = (_a = dev_tunnels_ssh_1.ECDsa.curves.find((c) => c.name === jwk.crv)) === null || _a === void 0 ? void 0 : _a.oid;
        if (!curveOid) {
            throw new Error(`Unknown EC curve: ${jwk.crv}`);
        }
        const parseBigInt = JsonWebKeyFormatter.parseBigInt;
        const ec = {
            curve: { name: jwk.crv, oid: curveOid },
            x: parseBigInt(jwk.x),
            y: parseBigInt(jwk.y),
        };
        if (jwk.d) {
            ec.d = parseBigInt(jwk.d);
        }
        return ec;
    }
    static formatBigInt(value, length) {
        return JsonWebKeyFormatter.base64UrlEncode(value.toBytes({ unsigned: true, length }));
    }
    static parseBigInt(value) {
        return dev_tunnels_ssh_1.BigInt.fromBytes(Buffer.from(value, 'base64'), { unsigned: true });
    }
    static base64UrlEncode(data) {
        // JWK format uses base64-url-encoding, which is base64 but with 2 substituted characters.
        // (Note Buffer's base64 DECODING implicitly supports this format.)
        return data
            .toString('base64')
            .replace(/=+$/g, '')
            .replace(/\+/g, '-')
            .replace(/\//g, '_');
    }
}
exports.JsonWebKeyFormatter = JsonWebKeyFormatter;
//# sourceMappingURL=jsonWebKeyFormatter.js.map